"""
skd Templatetags specific to listview

Every tag method is called with the specific template object and a context
defined as "list_context" class variable in the specific list view.
"""

from django import template
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _

from lib.actionlog import actionlogize, get_actionobject

register = template.Library()

@register.assignment_tag()
def get_row_title(template_object, context):

    """Get row title

    The returned string is used for each row as a tooltip.
    """

    object_type = type(template_object).__name__

    if object_type == "UserInGroup":

        if context == "user":

            return template_object.group.comment

        elif context == "usergroup":

            return template_object.user.comment

    if object_type == "HostInGroup":

        if context == "hostgroup":

            return template_object.host.comment

    elif object_type == "UserGroupInHostGroup":

        if context == "hostgroup":
            return template_object.usergroup.comment

        elif context == "usergroup":

            return template_object.hostgroup.comment

    else:

        return template_object.comment

@register.assignment_tag()
def get_row_columns(template_object, context):

    """Get column data

    Returns the data that should be displayed in the template for each row
    """

    object_type = type(template_object).__name__

    if object_type == "User":

        return [
            template_object.name,
            template_object.fullname
        ]

    elif object_type == "UserKey":

        return [
            template_object.name
        ]

    elif object_type == "UserMap":

        return [
            template_object.host.name,
            template_object.username
        ]

    elif object_type == "UserInGroup":

        if context == "user":

            return [
                template_object.group.name
            ]

        elif context == "usergroup":

            return [
                template_object.user.name
            ]

    elif object_type == "UserGroup":

        return [
            template_object.name
        ]

    elif object_type == "Host":

        return [
            template_object.name,
            template_object.fqdn,
            template_object.user
        ]

    elif object_type == "HostGroup":

        return [
            template_object.name
        ]

    elif object_type == "HostInGroup":

        if context == "host":

            return [template_object.group.name]

        elif context == "hostgroup":

            return [
                template_object.host.name,
                template_object.host.fqdn,
                template_object.host.user
            ]

    elif object_type == "UserGroupInHostGroup":

        if context == "hostgroup":

            return [
                template_object.usergroup.name
            ]

        elif context == "usergroup":

            return [
                template_object.hostgroup.name
            ]

    elif object_type == "ActionLog":

        return [
            template_object.timestamp,
            template_object.user,
            _(actionlogize(template_object.action))
        ]

    return None


@register.assignment_tag(takes_context=True)
def get_row_actions(context, template_object, view_context):

    """Get actions for a row

    get custom actions per each row. Returns a list of dictionaries having
    the keys "title" and "url".
    """

    object_type = type(template_object).__name__

    user_perms = context["user"].get_group_permissions()
    retval = []

    if object_type == "User":

        if "skd.change_user" in user_perms:

            retval.append(
                {
                    "title": _("Edit"),
                    "url": reverse(
                        "users_edit",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.delete_user" in user_perms:

            retval.append(
                {
                    "title": _("Delete"),
                    "url": reverse(
                        "users_delete",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.list_users_keys" in user_perms:

            retval.append(
                {
                    "title": _("Keys"),
                    "url": reverse(
                        "users_keys_list",
                        kwargs={
                            "user": template_object.id
                        }
                    )
                }
            )

        if "skd.list_usermaps" in user_perms:

            retval.append(
                {
                    "title": _("Mapping"),
                    "url": reverse(
                        "users_map_list",
                        kwargs={
                            "user": template_object.id
                        }
                    )
                }
            )

        if "skd.list_usergroups" in user_perms:

            retval.append(
                {
                    "title": _("Groups"),
                    "url": reverse(
                        "users_usergroups_list",
                        kwargs={
                            "user": template_object.id
                        }
                    )
                }
            )

    elif object_type == "UserKey":

        if "skd.change_userkey" in user_perms:

            retval.append(
                {
                    "title": _("Edit"),
                    "url": reverse(
                        "users_keys_edit",
                        kwargs={
                            "user": template_object.user.id,
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.delete_userkey" in user_perms:

            retval.append(
                {
                    "title": _("Delete"),
                    "url": reverse(
                        "users_keys_delete",
                        kwargs={
                            "user": template_object.user.id,
                            "pk": template_object.id
                        }
                    )
                }
            )

    elif object_type == "UserMap":

        if "skd.change_usermap" in user_perms:

            retval.append(
                {
                    "title": _("Edit"),
                    "url": reverse(
                        "users_map_edit",
                        kwargs={
                            "user": template_object.user.id,
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.delete_usermap" in user_perms:

            retval.append(
                {
                    "title": _("Delete"),
                    "url": reverse(
                        "users_map_delete",
                        kwargs={
                            "user": template_object.user.id,
                            "pk": template_object.id
                        }
                    )
                }
            )

    elif object_type == "UserInGroup":

        if "skd.delete_useringroup" in user_perms:

            if view_context == "user":

                retval.append(
                    {
                        "title": _("Unassign"),
                        "url": reverse(
                            "users_usergroups_unassign",
                            kwargs={
                                "user": template_object.user.id,
                                "pk": template_object.id
                            }
                        )
                    }
                )

            elif view_context == "usergroup":

                retval.append(
                    {
                        "title": _("Unassign"),
                        "url": reverse(
                            "usergroups_users_unassign",
                            kwargs={
                                "usergroup": template_object.group.id,
                                "pk": template_object.user.id
                            }
                        )
                    }
                )

    elif object_type == "UserGroup":

        if "skd.change_usergroup" in user_perms:

            retval.append(
                {
                    "title": _("Edit"),
                    "url": reverse(
                        "usergroups_edit",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.delete_usergroup" in user_perms:

            retval.append(
                {
                    "title": _("Delete"),
                    "url": reverse(
                        "usergroups_delete",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.list_users_in_usergroups" in user_perms:

            retval.append(
                {
                    "title": _("Members"),
                    "url": reverse(
                        "usergroups_users_list",
                        kwargs={
                            "usergroup": template_object.id
                        }
                    )
                }
            )

        if "skd.list_usergroups_in_hostgroups" in user_perms:

            retval.append(
                {
                    "title": _("Assigned hostgroups"),
                    "url": reverse(
                        "usergroups_hostgroups_list",
                        kwargs={
                            "usergroup": template_object.id
                        }
                    )
                }
            )

    elif object_type == "Host":

        if "skd.change_host" in user_perms:

            retval.append(
                {
                    "title": _("Edit"),
                    "url": reverse(
                        "hosts_edit",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.delete_host" in user_perms:

            retval.append(
                {
                    "title": _("Delete"),
                    "url": reverse(
                        "hosts_delete",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.list_hosts_in_hostgroups":

            retval.append(
                {
                    "title": _("Groups"),
                    "url": reverse(
                        "hosts_hostgroups_list",
                        kwargs={
                            "host": template_object.id
                        }
                    )
                }
            )

        if "skd.setup_host" in user_perms:

            retval.append(
                {
                    "title": _("Setup"),
                    "url": reverse(
                        "hosts_setup",
                        kwargs={
                            "host": template_object.id
                        }
                    )
                }
            )

    elif object_type == "HostGroup":

        if "skd.change_hostgroup" in user_perms:

            retval.append(
                {
                    "title": _("Edit"),
                    "url": reverse(
                        "hostgroups_edit",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.delete_hostgroup" in user_perms:

            retval.append(
                {
                    "title": _("Delete"),
                    "url": reverse(
                        "hostgroups_delete",
                        kwargs={
                            "pk": template_object.id
                        }
                    )
                }
            )

        if "skd.list_hosts_in_hostgroups" in user_perms:

            retval.append(
                {
                    "title": _("Members"),
                    "url": reverse(
                        "hostgroups_hosts_list",
                        kwargs={
                            "hostgroup": template_object.id
                        }
                    )
                }
            )

        if "skd.list_usergroups_in_hostgroups" in user_perms:

            retval.append(
                {
                    "title": _("Assigned usergroups"),
                    "url": reverse(
                        "hostgroups_usergroups_list",
                        kwargs={
                            "hostgroup": template_object.id
                        }
                    )
                }
            )

    elif object_type == "HostInGroup":

        if "skd.delete_hostingroup" in user_perms:

            retval.append(
                {
                    "title": _("Unassign"),
                    "url": reverse(
                        "hosts_hostgroups_unassign",
                        kwargs={
                            "pk": template_object.id,
                            "host": template_object.host.id
                        }
                    )
                }
            )

    elif object_type == "UserGroupInHostGroup":

        if "skd.delete_usergroupinhostgroup" in user_perms:

            if view_context == "hostgroup":

                return [
                    {
                        "title": _("Unassign"),
                        "url": reverse(
                            "hostgroups_usergroups_unassign",
                            kwargs={
                                "hostgroup": template_object.hostgroup.id,
                                "pk": template_object.usergroup.id
                            }
                        )
                    }
                ]

            elif view_context == "usergroup":

                return [
                    {
                        "title": _("Unassign"),
                        "url": reverse(
                            "usergroups_hostgroups_unassign",
                            kwargs={
                                "usergroup": template_object.usergroup.id,
                                "pk": template_object.id
                            }
                        )
                    }
                ]

    elif object_type == "ActionLog":

        url = get_actionobject(
            template_object.action,
            template_object.objectid,
            template_object.objectid2,
            user_perms
        )

        if url:

            return [
                {
                    "title": _("Reference"),
                    "url": url
                }
            ]

    return retval